% Script for the simulation study of ``Some non-asymptotic results on resampling in high dimension, II: multiple tests'' by Sylvain Arlot, Gilles Blanchard and Etienne Roquain
% This is a demo script for performing a similar simulation study with
% other parameters
% 
%
% Copyright (C) 2009 Sylvain Arlot, Gilles Blanchard and Etienne Roquain
%
%   This program is free software: you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
%   along with this program.  If not, see <http://www.gnu.org/licenses/>
% 

%%%%%%%%%%%%%%%%%%%% Parameters having to be tuned 

%%%% Printing directory
print_dir = 'tmp/A_Fig3_';

%%%% Printing format
z_error_bars=0; % put error bars on the figures (1 = yes ; 0 = no)

z_eps=1;% print figures as eps
z_pdf=0;% print figures as pdf

%%%% Other important parameters 
z_calc=1;%%% 1 = make again the computations and save ; 0 = load previouly made computations
nb_repet=50;%%%% number of samples considered
var_bandwidth=0;% 1= the kernel bandwidth is variable (and the dynamic range is constant equal to its first value) ; 
%0 = the dynamic range of the mean is variable (and the bandwidth is constant equal to its first value)

%%%%%%%%%%% Which computations ?
%%%% List of thresholds to be computed (single step)
thresholds_list_stepone={'bonf','conc','conc_bonf','quant_bonf','quant_conc','ideal','single'};
%%%% List of thresholds to be computed (step-down)
thresholds_list_sd={'sd_bonf','sd_quant_uncent','hybrid'};

%%%%%%%%%%% Which plots ? (maximum 12 curves per plot ; otherwise, increase the length of line_style_list)
%%%% Plot of the average threshold: list of the thresholds
thresholds_list_avge={'quant_bonf','sd_quant_uncent','hybrid'};%{'conc_bonf','quant_bonf','sd_bonf','sd_quant_uncent','hybrid','bonf','ideal'};
%
%%%% Plot of the average power: list of the thresholds
thresholds_list_pow={'quant_bonf','sd_quant_uncent','hybrid'};%{'conc_bonf','quant_bonf','sd_bonf','sd_quant_uncent','hybrid','bonf','ideal'};
%
%%%% Plot of the average number of steps: list of the thresholds
thresholds_list_nb_step={'sd_quant_uncent','hybrid'};%{'sd_bonf','sd_quant_uncent','hybrid'};
%
%%%% Plot of the average power at early steps (for step-down): 
%%% list of the thresholds
thresholds_list_pow_earl_stop={'sd_quant_uncent','hybrid'};
%%% list of the steps
steps_list=[1 2 3];%%% Do as for Figure 3
%
%
%%%% Line style list
line_style_list={'k-','k:','k--','k--x','k:x','k-.','k-v','k--s','k:s','k--^','k:^','k-.^'};

%%%%%%%%%%%%%%%%%%%% Constant parameters
%%% Generation of data
nb_obs=1000;%%% number of observations in each sample
dim=128*128;%%% dimension of each observation (should be a square)
data_type='gauss_texture';%%% nature of the noise (see gener_data.m)
filter='exp(d^2)';%%% nature of correlations (see gener_data.m)
bandwidth_vect=(0:2:40);%%% vector of kernel bandwidth (see gener_data.m)
sigma_gener=1;%%% noise level 
mean_vec='zero';%%% type of mean vector (see mean_vector.m for adding new kinds of mean vectors)
if strcmp(mean_vec,'exponential')
    dynamic_range_vect=(5:5:30);%%% vector of dynamic range for the mean, if mean_vec='exponential' (see mean_vector.m)
else
    dynamic_range_vect=0;
end%if strcmp(mean_vec,'exponential')

%%% Analysis of data
level=0.05;
type='sup_two_sided';
data_law='Gaussian';
sigma_known=sigma_gener;
weights='Rad';
nb_mcmc=1000;
deltacb=0.1;
deltaq=0.1;
alpha0_mult=0.9;

%%%%% Precompute useful constants
BW=B_W(nb_obs,weights);
CW=C_W(nb_obs,weights);
gamma=1-2*binoinv(deltaq*level*alpha0_mult/2,nb_obs,0.5)/(nb_obs);

%%%% Other parameters
nb_simu=1000; % number of data samples generated for estimating the 'ideal' threshold
fontsize=14; %Font size for the graphs
linewidth=2; %Line width for the graphs


%%%% This automatically adds to thresholds_list_stepone what is needed for
%%%% the step-down thresholds of thresholds_list_sd
th_l=cell(size(thresholds_list_sd));
for i_t=1:numel(thresholds_list_sd)
    s=thresholds_list_sd{i_t};
    if strcmp(s,'hybrid')
        th_l{i_t}='quant_bonf';
    else
        th_l{i_t}=s(4:end);
    end
end%for i_t=1:numel(thresholds_list_sd)
thresholds_list_stepone = union(thresholds_list_stepone, th_l);

%%%% Initialization of the vector of the varying parameter
if var_bandwidth
    param_vect=bandwidth_vect;
    % if the dynamic range should not vary, take it constant equal to the first value of the list
    dynamic_range=dynamic_range_vect(1);
else
    param_vect=dynamic_range_vect;
    % if the bandwidth should not vary, take it constant equal to the first value of the list
    bandwidth=bandwidth_vect(1);
end%if var_bandwidth

i_fig=0;

%%%%%%%%% Compute everything
if z_calc
%%%%% Make the computations
    fprintf('Starting computations\n');
    tic;
    %%% The loops go down in order to preallocate memory in struct 'result'
    for i_p=numel(param_vect):-1:1
        if var_bandwidth
            bandwidth=param_vect(i_p);
        else
            dynamic_range=dynamic_range_vect(i_p);
        end%if var_bandwidth

%%%            fprintf('Bandwidth = %d ; Dynamic range = %d : Computing ideal threshold \n',bandwidth,dynamic_range);
        %Compute ideal threshold (if needed)
        if max(strcmp(thresholds_list_stepone,'ideal'))
            ideal=ideal_threshold(nb_obs,dim,'data_type',data_type,'filter',filter,'bandwidth',bandwidth,'sigma',sigma_gener,'nb_simu',nb_simu,'type',type);
            result_ideal(i_p)=ideal;
        end%if max(strcmp(thresholds_list_stepone,'ideal'))

        for i_sample=nb_repet:-1:1
            fprintf('Sample number %d over %d \n',nb_repet-i_sample+1,nb_repet);
            %%% Generates data sample
            Y=gener_data(nb_obs,dim,'data_type',data_type,'filter',filter,'bandwidth',bandwidth,'mean',mean_vec,'sigma',sigma_gener,'dynamic_range',dynamic_range);


            %%%%% Start with the one-step thresholds (will also be used for
            %%%%% step-down thresholds)

            res=thresholds(Y,'level',level,'type',type,'data_law',data_law,'sigma',sigma_known,'weights',weights,...
                'nb_mcmc',nb_mcmc,'deltacb',deltacb,'deltaq',deltaq,'alpha0_mult',alpha0_mult,'BW',BW,'CW',CW,'gamma',gamma,...
                'thresholds_list',thresholds_list_stepone);
            if max(strcmp(thresholds_list_stepone,'ideal'))
                res.ideal=ideal;
            end%if max(strcmp(thresholds_list_stepone,'ideal'))
            res.bandwidth=bandwidth;
            res.dynamic_range=dynamic_range;

            %%% Computes empirical mean
            Empmat = mean(Y,2);
            %%% Record number of true/false positives/negatives for each threshold
            for i_thr=1:numel(thresholds_list_stepone)
                [res.([ thresholds_list_stepone{i_thr} '_FP']) , res.([ thresholds_list_stepone{i_thr} '_TP']) , res.([ thresholds_list_stepone{i_thr} '_FN']) , ...
                    res.([ thresholds_list_stepone{i_thr} '_TN']) ]=Power_level(res.(thresholds_list_stepone{i_thr}),Empmat',mean_vec,type);
                res.([ thresholds_list_stepone{i_thr} '_power'])=(res.([ thresholds_list_stepone{i_thr} '_TP']))/...
                    ((res.([ thresholds_list_stepone{i_thr} '_TP']))+(res.([ thresholds_list_stepone{i_thr} '_FN'])));
            end%for i_thr=1:numel(thresholds_list_stepone)

            %%%%% Now compute the step-down thresholds 
            res_sd=thresholds_sd(Y,'level',level,'type',type,'data_law',data_law,'sigma',sigma_known,'weights',weights,...
                'thresholds_list',thresholds_list_sd,'results_stepone',res,...
                'nb_mcmc',nb_mcmc,'deltacb',deltacb,'deltaq',deltaq,'alpha0_mult',alpha0_mult,'BW',BW,'CW',CW,'gamma',gamma);

            for i_thr=1:numel(thresholds_list_sd)
                %%% Store the computed thresholds in the struct 'res'
                res.(thresholds_list_sd{i_thr})=res_sd.(thresholds_list_sd{i_thr});

                %%% Record number of true/false positives/negatives for
                %%% each threshold at each stage of the algorithm

                %%% First, Retrieve the list of values of the threshold
                th_v=res.(thresholds_list_sd{i_thr});
                %%% Then, for each element of the list, evaluate the
                %%% number of true/false positives/negatives
                n_v=numel(th_v);
                FP_v=zeros(1,n_v);
                TP_v=zeros(1,n_v);
                FN_v=zeros(1,n_v);
                TN_v=zeros(1,n_v);
                for i_v=1:n_v
                    [FP_v(i_v) TP_v(i_v) FN_v(i_v) TN_v(i_v)]=Power_level(th_v(i_v),Empmat',mean_vec,type);
                end%for i_v=1:n_v
                %%% Store the four vectors FP_v, TP_v, FN_v, TN_v in res
                res.([ thresholds_list_sd{i_thr} '_FP'])=FP_v;
                res.([ thresholds_list_sd{i_thr} '_TP'])=TP_v;
                res.([ thresholds_list_sd{i_thr} '_FN'])=FN_v;
                res.([ thresholds_list_sd{i_thr} '_TN'])=TN_v;
                res.([ thresholds_list_sd{i_thr} '_power'])=(TP_v ./ (TP_v+FN_v));
            end%for i_thr=1:numel(thresholds_list)

            result(i_p,i_sample)=res;
            toc;
        end%for i_sample=1:nb_repet
    end%for i_p=numel(param_vect):-1:1
    save([print_dir 'data.mat'],'result');
    if max(strcmp(thresholds_list_stepone,'ideal'))
        save([print_dir 'data_ideal.mat'],'result_ideal','bandwidth_vect','dynamic_range_vect','var_bandwidth');
    end%if max(strcmp(thresholds_list_stepone,'ideal'))
else
    load([print_dir 'data.mat'],'result');
    if max(strcmp(thresholds_list_stepone,'ideal'))
        try
            load([print_dir 'data_ideal.mat'],'result_ideal','bandwidth_vect','dynamic_range_vect','var_bandwidth');
        catch
            fprintf(['error when loading ' print_dir 'data_ideal.mat']);
        end
    end%if max(strcmp(thresholds_list_stepone,'ideal'))
end%if z_calc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% Analyse data

%%% Recover list of bandwidths/dynamic ranges considered
if var_bandwidth
    param_list=[result.bandwidth];
else
    param_list=[result.dynamic_range];
end
param_vect=unique(param_list);

%%% List of all thresholds of interest 
thresholds_list_all=union(union(thresholds_list_avge,thresholds_list_pow),union(thresholds_list_nb_step,thresholds_list_pow_earl_stop));

for i_thr=1:numel(thresholds_list_all)
    s=thresholds_list_all{i_thr};
    
    %%% Preallocate memory
    eval([ s '_list = zeros(1,numel(param_list));']);
    eval([ s '_nb_step_list = zeros(1,numel(param_list));']);
    eval([ s '_power_list = zeros(1,numel(param_list));']);
    for i_s=1:numel(steps_list)
        eval([ s '_power_stop' num2str(steps_list(i_s)) '_list = zeros(1,numel(param_list));']);
    end%for i_s=1:numel(steps_list)

    %%% Recover values of the thresholds (taking the final value for
    %%% step-down methods), the number of steps needed and the power
    for i_p=1:numel(param_list)
        %%% values of the thresholds
        eval([ s '_list(i_p) = result(i_p).' s '(end);']);
        %%% number of steps (only interesting for step-down methods)
        eval([ s '_nb_step_list(i_p) = numel(result(i_p).' s ');']);
        %%% power of the thresholds
        eval([ s '_power_list(i_p) = result(i_p).' s '_power(end);']);
        %%% power of the thresholds stopped earlier
        for i_s=1:numel(steps_list)
            eval([ s '_power_stop' num2str(steps_list(i_s)) '_list(i_p) = result(i_p).' s '_power(min(end,steps_list(i_s)));']);
        end%for i_s=1:numel(steps_list)
    end%for i_p=1:numel(param_list)

    %%% Compute mean and standard-deviation given the value of the
    %%% parameter (bandwidth or dynamic range)
    %%% Preallocate memory
    eval([ s '_mean = zeros(size(param_vect));']);
    eval([ s '_err = zeros(size(param_vect));']);
    eval([ s '_nb_step_mean = zeros(size(param_vect));']);
    eval([ s '_nb_step_err = zeros(size(param_vect));']);
    eval([ s '_power_mean = zeros(size(param_vect));']);
    eval([ s '_power_err = zeros(size(param_vect));']);
    for i_s=1:numel(steps_list)
        eval([ s '_power_stop' num2str(steps_list(i_s)) '_mean = zeros(1,numel(param_vect));']);
        eval([ s '_power_stop' num2str(steps_list(i_s)) '_err = zeros(1,numel(param_vect));']);
    end%for i_s=1:numel(steps_list)

    for i_p=1:numel(param_vect)
        par=param_vect(i_p);
        %%% value of the threshold
        vec=eval([ s '_list ( param_list == par )']);
        eval([ s '_mean (i_p) = mean(vec);']);
        eval([ s '_err (i_p) = std(vec)/sqrt(numel(vec));']);
        %%% power of the threshold
        vec=eval([ s '_power_list ( param_list == par )']);
        eval([ s '_power_mean (i_p) = mean(vec);']);
        eval([ s '_power_err (i_p) = std(vec)/sqrt(numel(vec));']);
        %%% number of steps (only interesting for step-down methods)
        vec=eval([ s '_nb_step_list ( param_list == par )']);
        eval([ s '_nb_step_mean (i_p) = mean(vec);']);
        eval([ s '_nb_step_err (i_p) = std(vec)/sqrt(numel(vec));']);
        %%%
        for i_s=1:numel(steps_list)
            vec=eval([ s '_power_stop' num2str(steps_list(i_s)) '_list ( param_list == par )']);
            eval([ s '_power_stop' num2str(steps_list(i_s)) '_mean (i_p) = mean(vec);']);
            eval([ s '_power_stop' num2str(steps_list(i_s)) '_err (i_p) = std(vec)/sqrt(numel(vec));']);
        end%for i_s=1:numel(steps_list)
    end%for i_p=1:numel(param_vect)
end%for i_thr=1:numel(thresholds_list_all)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% Plot the data

if var_bandwidth
    xlabel_str='Convolution kernel width (pixels)';
else
    xlabel_str='Signal range (dB)';
end

%%%% Plot of the average threshold
i_fig=i_fig+1;
figure(i_fig);
newplot;
hold on;
set(gca,'FontSize',fontsize);
legend_str='';
for i_thr=1:numel(thresholds_list_avge)
    if z_error_bars
    %%% with error bars
        eval(['errorbar(param_vect, ' thresholds_list_avge{i_thr} '_mean , ' thresholds_list_avge{i_thr} '_err , ''' line_style_list{i_thr} ''' , ''Linewidth'', linewidth )']);
    else
    %%% without error bars
        eval(['plot(param_vect, ' thresholds_list_avge{i_thr} '_mean , ''' line_style_list{i_thr} ''' , ''Linewidth'', linewidth )']);
    end%if z_error_bars
    legend_str=[legend_str ' ''' abr2nam(thresholds_list_avge{i_thr})  ''', '];
end
xlabel(xlabel_str);
ylabel('Av. threshold');
title({['K=' num2str(sqrt(dim)) 'x' num2str(sqrt(dim)) ', n=' num2str(nb_obs) ', level ' num2str(level * 100) '%']});
eval(['legend(' legend_str ' ''Location'',''Best'' )']);
hold off;

%%%% Print the plot
if z_eps
    print('-f','-r600','-depsc2',[print_dir 'Avge-thr_' data_type '.eps']);
end
if z_pdf
    print('-f','-r600','-dpdf',[print_dir 'Avge-thr_' data_type '.pdf']);
end

%%%% Plot of the average power
i_fig=i_fig+1;
figure(i_fig);
newplot;
hold on;
set(gca,'FontSize',fontsize);
legend_str='';
for i_thr=1:numel(thresholds_list_pow)
    if z_error_bars
    %%% with error bars
        eval(['errorbar(param_vect, ' thresholds_list_pow{i_thr} '_power_mean , ' thresholds_list_pow{i_thr} '_power_err , ''' line_style_list{i_thr} ''' , ''Linewidth'', linewidth )']);
    else
    %%% without error bars
        eval(['plot(param_vect, ' thresholds_list_pow{i_thr} '_power_mean , ''' line_style_list{i_thr} ''' , ''Linewidth'', linewidth )']);
    end%if z_error_bars
    legend_str=[legend_str ' ''' abr2nam(thresholds_list_pow{i_thr})  ''', '];
end
xlabel(xlabel_str);
ylabel('Power');
title({['K=' num2str(sqrt(dim)) 'x' num2str(sqrt(dim)) ', n=' num2str(nb_obs) ', level ' num2str(level * 100) '%']});
eval(['legend(' legend_str ' ''Location'',''Best'' )']);
hold off;

%%%% Print the plot
if z_eps
    print('-f','-r600','-depsc2',[print_dir 'Power_' data_type '.eps']);
end
if z_pdf
    print('-f','-r600','-dpdf',[print_dir 'Power_' data_type '.pdf']);
end

%%%% Plot of the average number of steps
i_fig=i_fig+1;
figure(i_fig);
newplot;
hold on;
set(gca,'FontSize',fontsize);
legend_str='';
for i_thr=1:numel(thresholds_list_nb_step)
    if z_error_bars
    %%% with error bars
        eval(['errorbar(param_vect, ' thresholds_list_nb_step{i_thr} '_nb_step_mean , ' thresholds_list_nb_step{i_thr} '_nb_step_err , ''' line_style_list{i_thr} ''' , ''Linewidth'', linewidth )']);
    else
    %%% without error bars
        eval(['plot(param_vect, ' thresholds_list_nb_step{i_thr} '_nb_step_mean , ''' line_style_list{i_thr} ''' , ''Linewidth'', linewidth )']);
    end%if z_error_bars
    legend_str=[legend_str ' ''' abr2nam(thresholds_list_nb_step{i_thr})  ''', '];
end
xlabel(xlabel_str);
ylabel('Av. number of iterations');
title({['K=' num2str(sqrt(dim)) 'x' num2str(sqrt(dim)) ', n=' num2str(nb_obs) ', level ' num2str(level * 100) '%']});
eval(['legend(' legend_str ' ''Location'',''Best'' )']);
hold off;

%%%% Print the plot
if z_eps
    print('-f','-r600','-depsc2',[print_dir 'Nb_steps_' data_type '.eps']);
end
if z_pdf
    print('-f','-r600','-dpdf',[print_dir 'Nb_steps_' data_type '.pdf']);
end

%%%% Plot of the average power at early steps (for step-down): 
%thresholds_list_pow_earl_stop={'sd_quant_uncent','hybrid'};
%%% list of the steps
%steps_list=[1 2 3];
%
i_fig=i_fig+1;
figure(i_fig);
newplot;
hold on;
set(gca,'FontSize',fontsize);
legend_str='';
for i_thr=1:numel(thresholds_list_pow_earl_stop)
    for i_s=1:numel(steps_list)
        i_line=i_thr+numel(thresholds_list_pow_earl_stop)*(i_s - 1);
        if z_error_bars
        %%% with error bars
            eval(['errorbar(param_vect, ' thresholds_list_pow_earl_stop{i_thr} '_power_stop' num2str(steps_list(i_s)) '_mean , ' thresholds_list_pow_earl_stop{i_thr} ...
                '_power_stop' num2str(steps_list(i_s)) '_err , ''' line_style_list{i_line} ''' , ''Linewidth'', linewidth )']);
        else
        %%% without error bars
            eval(['plot(param_vect, ' thresholds_list_pow_earl_stop{i_thr} '_power_stop' num2str(steps_list(i_s)) '_mean , ''' line_style_list{i_line} ''' , ''Linewidth'', linewidth )']);
        end%if z_error_bars
        legend_str=[legend_str ' ''' abr2nam(thresholds_list_pow_earl_stop{i_thr}) ', t=' num2str(steps_list(i_s))  ''', '];
    end%for i_s=1:numel(steps_list)
end%for i_thr=1:numel(thresholds_list_pow_earl_stop)
xlabel(xlabel_str);
ylabel('Av. power');
title({['K=' num2str(sqrt(dim)) 'x' num2str(sqrt(dim)) ', n=' num2str(nb_obs) ', level ' num2str(level * 100) '%']});
eval(['legend(' legend_str ' ''Location'',''Best'' )']);
hold off;

%%%% Print the plot
if z_eps
    print('-f','-r600','-depsc2',[print_dir 'Power-early_' data_type '.eps']);
end
if z_pdf
    print('-f','-r600','-dpdf',[print_dir 'Power-early_' data_type '.pdf']);
end