function Y = gener_data(nb_obs,dim,varargin)
%
% This function computes the thresholds defined in the paper ``Some non-asymptotic results on resampling in high dimension, II: multiple tests'' by Sylvain Arlot, Gilles Blanchard and Etienne Roquain
%
% Copyright (C) 2009 Sylvain Arlot, Gilles Blanchard and Etienne Roquain
%
% IN : 
% 
%	'nb_obs' : number of observations
%	'dim'    : dimension 
%	
% followed by a list of arguments of the form 'property',value
%      (if not given the default is used)
%
%	'data_type' : which kind of data are to be generated ?
%		currently available:
%		'gauss_texture' (default)
%
%	'filter' : option for the 'gauss_texture' case (shape of the filter)
%		default 'exp(d^2)'
%
%	'bandwidth' : option for the 'gauss_texture' case (width of the filter)
%		default 30
%
%   'mean' : different kinds of mean vector are currently available
%       'zero' (default) : mu=0
%       'linear'        : setting of Section 4.3 : half of the means are
%           zero ; the other half are increasing linearly from
%           (5/16)*(2 sided Bonferroni threshold) to 20*(2 sided Bonferroni
%           threshold) (assuming level=0.05)
%       'exponential'  : setting of Section 4.4
%
%   'dynamic_range' : option for the 'exponential' mean setting (default
%   20)
%
%   'sigma' : value of the (constant) standard deviation (default 1)
%
%  OUT : 'Y' is a dim*nb_obs data matrix 
%

opt = propertylist2struct(varargin{:});

[opt, isdefault] = set_defaults(opt, 'filter','exp(d^2)', ...
                    'bandwidth',30,...
				    'data_type','gauss_texture',...
                    'sigma',1,...
                    'mean','zero',...
                    'dynamic_range',20); 


switch (opt.data_type)
	case 'gauss_texture'
% generates a stationnary Gaussian process on the 2D torus
%  as a nobs x d vector; d should be a squared integer = edge^2
% 
% it is obtained by convoluting a standard iid Gaussian by
% some kernel; filter should be the 2D fft of the convolution kernel
%
		ed = floor(sqrt(dim));

		if ( dim ~= ed*ed) %#ok<ALIGN>
			error('Dimension must be a square.');
        end
		

%%% define the filter to be used
        switch (opt.filter)
            case 'exp(d^2)'
                scal = zeros(ed,ed);
                scal(1,1)=1;
                if opt.bandwidth~=0
                    for k=1:ed; 
                      for l=1:ed; 
                        % x and y distance to the origin on the torus

                        di = min(k-1,ed-(k-1));
                        dj = min(l-1,ed-(l-1));

                        scal(k,l) = exp(-(di^2+dj^2)/(opt.bandwidth)^2); 
                      end;
                    end;
                end%if opt.bandwidth~=0
                scal=scal/(sqrt(sum(sum(scal.^2))));
                scal=fft2(scal);
            otherwise
                scal=NaN*ones(ed,ed);
        end

		Y = zeros(dim,nb_obs); % reserve some memory: goes faster

		for k=1:nb_obs 
			img = real( ifft2( (randn(ed,ed) + i*randn(ed,ed) ).*scal))*ed;
			Y(:,k) = img(:);
        end

        %%% Multiply by standard deviation and add mean
        mu=mean_vector(opt.mean,dim,'dynamic_range',opt.dynamic_range,'nb_obs',nb_obs,'sigma',opt.sigma);%%%
        Y = repmat(mu, [nb_obs 1])' + opt.sigma*Y;

	otherwise
		Y=NaN*ones(dim,nb_obs);
end
