function result=thresholds_sd(Ymat,varargin)
%
% This function computes the step-down thresholds defined in the paper ``Some non-asymptotic results on resampling in high dimension, II: multiple tests'' by Sylvain Arlot, Gilles Blanchard and Etienne Roquain
%
% Copyright (C) 2009 Sylvain Arlot, Gilles Blanchard and Etienne Roquain
%
% result=thresholds_sd(Y,'level',level,'type',type,'data_law',data_law,'sigma',sigma,'weights',weights,'thresholds_list',thresholds_list,...
%   'results_stepone',results_stepone,'nb_mcmc',nb_mcmc,...
%   'deltacb',deltacb,'deltaq',deltaq,'alpha0_mult',alpha0_mult,'AW',AW,'DW',DW,'BW',BW,'CW',CW,'gamma',gamma);
% 
%
% IN : 
% 
%	'Ymat' : data matrix of size K x n. The columns are supposed to be iid. The multiple test looks for the rows having a mean different from zero.
%
% followed by a list of arguments of the form 'property',value
%      (if not given the default is used)
%
%	'level' : confidence level of the procedure
%                default 0,05
%	'type' : type of experiment (function phi);
%                currently available:
%                   'sup_one_sided'
%                   'sup_two_sided' (default)
%	'data_law' : assumption on the distribution of the data: 
%                currently available:
%                   'Gaussian' (default)
%                   'symmetric'
%	'sigma' : (known or estimated) value of phi(sigma)
%		default: estimates it with the function sigma_estim 
%	'weights' : distribution of the weights used for computing the concentration thresholds
%		currently available:
%		'Rad'    : Rad(1/2) weights (default)
%		'Rad(p)' : Bernoulli (p) weights
%		'Efr'    : Efron's bootstrap (n) weights
%		'Efr(M)' : Efron's bootstrap (M) weights
%		'Rho'    : Random hold-out (n/2) weights
%		'Rho(q)' : Random hold-out (q) weights
%		'Loo'    : Leave-one-out weights
%
%	'thresholds_list' : list of the thresholds to be computed 
%		largest list available : {'sd_bonf','sd_conc','sd_conc_bonf','sd_quant_bonf','sd_quant_conc','sd_quant','sd_quant_uncent','hybrid'}
%		default {'sd_bonf','sd_quant_bonf','sd_quant_uncent','hybrid'}
%
%	'results_stepone' : struct containing the thresholds computed at the first step
%		(default = compute it with thresholds.m)
%
%   'earlystop' : maximal number of steps for each step-down procedure
%   (1 = perform only step one; 2 or more = go into the step-down loop; Inf
%   = never stop the step-down loop before it naturally stops)
%       default = Inf
%
%	'nb_mcmc' : number of resamplings for each experiment
%		default 1000
%	'deltacb' : parameter delta used in threshold min(conc,bonf)
%		default 0.1 
%	'deltaq' : parameter delta used in the quantile threshold
%		default 0.1 
%	'alpha0_mult' : will be multiplied by the level to compute alpha_0 used in the quantile threshold
%		default 0.9
%	'AW' : value of the constant A_W (default = compute it with A_W.m)
%	'BW' : value of the constant B_W (default = compute it with B_W.m)
%	'CW' : value of the constant C_W (default = compute it with C_W.m)
%	'DW' : value of the constant D_W (default = compute it with D_W.m)
%	'gamma' : value of the constant gamma_n(alpha_0 delta) (default = compute it from the other parameters)
%
%  OUT : result is a struct variable with fields:
%
%	'parameters' : store all the above parameters for the record
%
%       plus a field per threshold appearing in thresholds_list, among :
%
%	'sd_bonf'       : List of thresholds with the Step-down Bonferroni procedure (= Holm's procedure)
%	'sd_conc'       : List of thresholds with the Step-down concentration procedure
%	'sd_conc_bonf'  : List of thresholds with the Step-down min(conc,bonf) procedure
%	'sd_quant_conc' : List of thresholds with the Step-down quantile procedure with 'conc' correction term
%	'sd_quant_bonf' : List of thresholds with the Step-down quantile procedure with 'bonf' correction term
%	'sd_quant'      : List of thresholds with the Step-down quantile procedure (without correction term) = 'centered symmetrization quantile'
%	'sd_quant_uncent' : List of thresholds with the Step-down uncentered quantile procedure 'classical symmetrization quantile'
%   'hybrid'        : List of thresholds with the Hybrid approach
%

opt = propertylist2struct(varargin{:});

[opt, isdefault] = set_defaults(opt, 'deltaq', 0.1, ...
				    'alpha0_mult', 0.9,...
				    'deltacb', 0.1,...
				    'level', 0.05,...
				     'type','sup_two_sided',...
				    'sigma',NaN,...
				    'nb_mcmc', 1000,...
				     'weights','Rad',...
			             'data_law','Gaussian',...
				     'AW',NaN,...
				     'BW',NaN,...
				     'CW',NaN,...
				     'DW',NaN,...
				     'gamma',NaN,...
				     'thresholds_list',{'sd_bonf','sd_quant_bonf','sd_quant_uncent','hybrid'},...
				     'results_stepone',struct('void',NaN),...
                     'earlystop',Inf);

if isnan(opt.sigma)
	opt.sigma=sigma_estim(Ymat,opt.type);
end

[opt.dim opt.nb_obs]=size(Ymat);

if (isnan(opt.AW) && strcmp(opt.data_law,'symmetric'))
	opt.AW=A_W(opt.nb_obs,opt.weights);
end

if (isnan(opt.BW) && strcmp(opt.data_law,'Gaussian'))
	opt.BW=B_W(opt.nb_obs,opt.weights);
end

if (isnan(opt.CW) && strcmp(opt.data_law,'Gaussian'))
	opt.CW=C_W(opt.nb_obs,opt.weights);
end

if (isnan(opt.DW) && strcmp(opt.data_law,'symmetric'))
	opt.DW=D_W(opt.nb_obs,opt.weights);
end

if isnan(opt.gamma)
	opt.gamma = 1-2*binoinv(opt.deltaq*opt.level*opt.alpha0_mult/2,opt.nb_obs,0.5)/(opt.nb_obs);
end

%%% Determine which thresholds have to be computed for the first step
th_list_stepone=opt.thresholds_list;
for i_t=1:numel(th_list_stepone)
    s=th_list_stepone{i_t};
    if strcmp(s,'hybrid')
        th_list_stepone{i_t}='quant_bonf';
    else
        th_list_stepone{i_t}=s(4:end);
    end
end%for i_t=1:numel(th_list_stepone)
if isfield(opt.results_stepone,'void')
    %%% Needs to perform the first step
    %%% Second, compute them via thresholds.m
    opt.results_stepone=thresholds(Ymat,'level',opt.level,'type',opt.type,'data_law',opt.data_law,'sigma',opt.sigma,'weights',opt.weights,'nb_mcmc',opt.nb_mcmc,...
        'deltacb',opt.deltacb,'deltaq',opt.deltaq,'alpha0_mult',opt.alpha0_mult,'BW',opt.BW,'CW',opt.CW,'gamma',opt.gamma,'thresholds_list',th_list_stepone);
end

result.parameters = opt; %store the used parameters in the results,
                         %that helps later

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%% Compute the absolute value (resp. the positive part) of the empirical mean
%%%%% These values will be compared to the thresholds by the multiple testing procedures
switch opt.type
    case 'sup_one_sided'
        phiEmpmean = max(mean(Ymat,2),0);
    case 'sup_two_sided'
        phiEmpmean = abs(mean(Ymat,2));
    otherwise
        phiEmpmean = NaN*abs(mean(Ymat,2));
end
        

%%%%% For each threshold in the list, perform the step-down algorithm
for i_t=1:numel(opt.thresholds_list)
    s_sone=th_list_stepone{i_t};
    s_sd=opt.thresholds_list{i_t};
    fprintf('Step one : %s ; Then, %s \n',s_sone,s_sd);
    %%% Perform first step (the threshold having already been computed)
    %%% Initialization
    coordinates_vect=ones(size(phiEmpmean));%indicates which coordinates are still in the set C_0
    i_step=1;
    %%% Value of the threshold
    th_list=[opt.results_stepone.(s_sone)];
    fprintf('Step %d : threshold = %d \n',i_step,th_list(i_step));
    %%% Which coordinates are still in the set C_1 ?
    coordinates_vect_new=( phiEmpmean <= th_list(i_step)) ;
    %%% Test whether some new coordinates have been rejected
    newreject = max( coordinates_vect ~= coordinates_vect_new);
    
    %%% Then, start the step-down loop (only if newreject is true)
    while (newreject && (i_step < opt.earlystop))
        %%% Update values before performing the new step
        coordinates_vect=coordinates_vect_new;
        i_step=i_step+1;
        
        %%% Value of the threshold
        if strcmp(s_sd,'hybrid')
            %%% Case of the hybrid approach : compute the quant_uncent
            %%% threshold at level opt.level * opt.alpha0_mult
            %%%
            res=thresholds(Ymat(coordinates_vect,:),'thresholds_list',{'quant_uncent'},'level',(opt.level)*(opt.alpha0_mult),'type',opt.type,'data_law',opt.data_law,'sigma',opt.sigma,'weights',opt.weights,'nb_mcmc',opt.nb_mcmc,...
               'deltacb',opt.deltacb,'deltaq',opt.deltaq,'alpha0_mult',opt.alpha0_mult,'AW',opt.AW,'DW',opt.DW,'BW',opt.BW,'CW',opt.CW,'gamma',opt.gamma);
            th_list(i_step)=min(th_list(i_step-1), res.quant_uncent);
        else
            %%% Other procedures : simply compute the s_sone threshold
            %%% at level opt.level
            %%%
            res=thresholds(Ymat(coordinates_vect,:),'thresholds_list',{s_sone},'level',opt.level,'type',opt.type,'data_law',opt.data_law,'sigma',opt.sigma,'weights',opt.weights,'nb_mcmc',opt.nb_mcmc,...
               'deltacb',opt.deltacb,'deltaq',opt.deltaq,'alpha0_mult',opt.alpha0_mult,'AW',opt.AW,'DW',opt.DW,'BW',opt.BW,'CW',opt.CW,'gamma',opt.gamma);
            th_list(i_step)=min(th_list(i_step-1), res.(s_sone));
        end%if strcmp(s_sd,'hybrid')

        fprintf('Step %d : threshold = %d \n',i_step,th_list(i_step));
        
        %%% Which coordinates are still in the set C_i ?
        coordinates_vect_new=( phiEmpmean <= th_list(i_step)) ;
        
        %%% Test whether some new coordinates have been rejected
        newreject = max( coordinates_vect ~= coordinates_vect_new);
    end%while newreject
    %%%
    %%% Save list of thresholds in 'result'
    result.(s_sd)=th_list;
end%for i_t=1:numel(th_list_stepone)

